# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/service_account_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import service_account_pb2 as nebius_dot_iam_dot_v1_dot_service__account__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/iam/v1/service_account_service.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a#nebius/iam/v1/service_account.proto\"\x94\x01\n\x1b\x43reateServiceAccountRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.iam.v1.ServiceAccountSpecR\x04spec\"*\n\x18GetServiceAccountRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"a\n\x1eGetServiceAccountByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\x9f\x01\n\x19ListServiceAccountRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12 \n\tpage_size\x18\x02 \x01(\x03H\x00R\x08pageSize\x88\x01\x01\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilterB\x0c\n\n_page_size\"\x94\x01\n\x1bUpdateServiceAccountRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.iam.v1.ServiceAccountSpecR\x04spec\"-\n\x1b\x44\x65leteServiceAccountRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"y\n\x1aListServiceAccountResponse\x12\x33\n\x05items\x18\x01 \x03(\x0b\x32\x1d.nebius.iam.v1.ServiceAccountR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xa3\x04\n\x15ServiceAccountService\x12Q\n\x06\x43reate\x12*.nebius.iam.v1.CreateServiceAccountRequest\x1a\x1b.nebius.common.v1.Operation\x12M\n\x03Get\x12\'.nebius.iam.v1.GetServiceAccountRequest\x1a\x1d.nebius.iam.v1.ServiceAccount\x12Y\n\tGetByName\x12-.nebius.iam.v1.GetServiceAccountByNameRequest\x1a\x1d.nebius.iam.v1.ServiceAccount\x12[\n\x04List\x12(.nebius.iam.v1.ListServiceAccountRequest\x1a).nebius.iam.v1.ListServiceAccountResponse\x12Q\n\x06Update\x12*.nebius.iam.v1.UpdateServiceAccountRequest\x1a\x1b.nebius.common.v1.Operation\x12Q\n\x06\x44\x65lete\x12*.nebius.iam.v1.DeleteServiceAccountRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBa\n\x14\x61i.nebius.pub.iam.v1B\x1aServiceAccountServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.service_account_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\032ServiceAccountServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _GETSERVICEACCOUNTBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETSERVICEACCOUNTBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETSERVICEACCOUNTBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETSERVICEACCOUNTBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _SERVICEACCOUNTSERVICE._options = None
  _SERVICEACCOUNTSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_CREATESERVICEACCOUNTREQUEST']._serialized_start=222
  _globals['_CREATESERVICEACCOUNTREQUEST']._serialized_end=370
  _globals['_GETSERVICEACCOUNTREQUEST']._serialized_start=372
  _globals['_GETSERVICEACCOUNTREQUEST']._serialized_end=414
  _globals['_GETSERVICEACCOUNTBYNAMEREQUEST']._serialized_start=416
  _globals['_GETSERVICEACCOUNTBYNAMEREQUEST']._serialized_end=513
  _globals['_LISTSERVICEACCOUNTREQUEST']._serialized_start=516
  _globals['_LISTSERVICEACCOUNTREQUEST']._serialized_end=675
  _globals['_UPDATESERVICEACCOUNTREQUEST']._serialized_start=678
  _globals['_UPDATESERVICEACCOUNTREQUEST']._serialized_end=826
  _globals['_DELETESERVICEACCOUNTREQUEST']._serialized_start=828
  _globals['_DELETESERVICEACCOUNTREQUEST']._serialized_end=873
  _globals['_LISTSERVICEACCOUNTRESPONSE']._serialized_start=875
  _globals['_LISTSERVICEACCOUNTRESPONSE']._serialized_end=996
  _globals['_SERVICEACCOUNTSERVICE']._serialized_start=999
  _globals['_SERVICEACCOUNTSERVICE']._serialized_end=1546
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/dns/v1/zone_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.dns.v1 import zone_pb2 as nebius_dot_dns_dot_v1_dot_zone__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n nebius/dns/v1/zone_service.proto\x12\rnebius.dns.v1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x18nebius/dns/v1/zone.proto2\xe1\x01\n\x0bZoneService\x12\x39\n\x03Get\x12\x1d.nebius.dns.v1.GetZoneRequest\x1a\x13.nebius.dns.v1.Zone\x12\x44\n\tGetByName\x12\".nebius.common.v1.GetByNameRequest\x1a\x13.nebius.dns.v1.Zone\x12I\n\x04List\x12\x1f.nebius.dns.v1.ListZonesRequest\x1a .nebius.dns.v1.ListZonesResponse\x1a\x06\xbaJ\x03\x64nsBW\n\x14\x61i.nebius.pub.dns.v1B\x10ZoneServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/dns/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.dns.v1.zone_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.dns.v1B\020ZoneServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/dns/v1'
  _ZONESERVICE._options = None
  _ZONESERVICE._serialized_options = b'\272J\003dns'
  _globals['_ZONESERVICE']._serialized_start=137
  _globals['_ZONESERVICE']._serialized_end=362
# @@protoc_insertion_point(module_scope)



# type: ignore
import logging

import pytest


@pytest.mark.asyncio
async def test_update_instance_v2() -> None:
    import grpc
    import grpc.aio

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.common.v1.operation_pb2 as operation_pb2
    import nebius.api.nebius.compute.v1.disk_pb2 as disk_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.api.nebius.compute.v1.disk_service_pb2 import (
        GetDiskRequest,
        UpdateDiskRequest,
    )
    from nebius.api.nebius.compute.v1.disk_service_pb2_grpc import (
        DiskServiceServicer,
        add_DiskServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE
    from nebius.base.version import version as sdk_version

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(DiskServiceServicer):
        async def Update(  # noqa: N802 — GRPC method
            self,
            request: UpdateDiskRequest,
            context: grpc.aio.ServicerContext[GetDiskRequest, disk_pb2.Disk],
        ) -> operation_pb2.Operation:
            assert request.metadata.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""
            got_mask = md.get("x-resetmask", "")
            assert (
                "metadata.(created_at.(nanos,seconds),labels.*,name,parent_id,"
                + "resource_version,updated_at.(nanos,seconds)),spec.("
                in got_mask
            )
            ua = md.get("user-agent", "")
            assert ua.startswith(
                f"a b c test nebius-python-sdk/{sdk_version} (python/3."
            )
            assert ua.endswith(" x y z")

            await context.send_initial_metadata(
                (
                    ("x-request-id", "some-req-id"),
                    ("x-trace-id", "some-trace-id"),
                )
            )

            ret = operation_pb2.Operation()
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_DiskServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    try:
        # Set up the client channel
        channel = Channel(
            domain=address,
            options=[
                (INSECURE, True),
                ("grpc.primary_user_agent", "a"),
                ("grpc.primary_user_agent", "b"),
                ("grpc.secondary_user_agent", "x"),
                ("grpc.secondary_user_agent", "y"),
            ],
            address_options={
                f"compute.localhost:{port}": [
                    ("grpc.primary_user_agent", "c"),
                    ("grpc.secondary_user_agent", "z"),
                ]
            },
            credentials=NoCredentials(),
            user_agent_prefix="test",
        )
        from nebius.aio.operation import Operation
        from nebius.api.nebius.compute.v1 import (
            DiskServiceClient,
            GetDiskRequest,
            UpdateDiskRequest,
        )

        client = DiskServiceClient(channel)
        upd = UpdateDiskRequest()
        upd.metadata.id = "foo-bar"
        req = client.update(upd)

        # Await response and metadata
        ret = await req
        assert isinstance(ret, Operation)
    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)


@pytest.mark.asyncio
async def test_update_list() -> None:
    import grpc
    import grpc.aio

    # Imports needed inside the test function
    from grpc.aio._metadata import Metadata

    import nebius.api.nebius.common.v1.operation_pb2 as operation_pb2
    import nebius.api.nebius.compute.v1.instance_pb2 as instance_pb2
    from nebius.aio.channel import Channel, NoCredentials
    from nebius.api.nebius.compute.v1.instance_service_pb2 import (
        GetInstanceRequest,
        UpdateInstanceRequest,
    )
    from nebius.api.nebius.compute.v1.instance_service_pb2_grpc import (
        InstanceServiceServicer,
        add_InstanceServiceServicer_to_server,
    )
    from nebius.base.options import INSECURE

    # Set up logging
    logging.basicConfig(level=logging.DEBUG)

    # Define a mock server class
    class MockInstanceService(InstanceServiceServicer):
        async def Update(  # noqa: N802 — GRPC method
            self,
            request: UpdateInstanceRequest,
            context: grpc.aio.ServicerContext[
                GetInstanceRequest, instance_pb2.Instance
            ],
        ) -> operation_pb2.Operation:
            assert request.metadata.id == "foo-bar"
            md = context.invocation_metadata()
            assert md is not None
            # Recreate metadata for ease of checking
            md = Metadata(*[v for v in md])
            assert md.get("x-idempotency-key", "") != ""
            mask = md.get("x-resetmask", "")
            assert (
                "metadata.("
                "created_at.(nanos,seconds),labels.*,name,parent_id,resource_version,"
                "updated_at.(nanos,seconds)"
                "),"
                "spec.(" in mask
            )

            await context.send_initial_metadata(
                (
                    ("x-request-id", "some-req-id"),
                    ("x-trace-id", "some-trace-id"),
                )
            )

            ret = operation_pb2.Operation()
            return ret

    # Randomly assign an IPv6 address and port for the server
    srv = grpc.aio.server()
    assert isinstance(srv, grpc.aio.Server)
    port = srv.add_insecure_port("[::]:0")
    add_InstanceServiceServicer_to_server(MockInstanceService(), srv)
    await srv.start()

    # Use the actual port assigned by the server
    address = f"localhost:{port}"

    channel = None
    try:
        # Set up the client channel
        channel = Channel(
            domain=address, options=[(INSECURE, True)], credentials=NoCredentials()
        )
        from nebius.aio.operation import Operation
        from nebius.api.nebius.compute.v1 import (
            AttachedFilesystemSpec,
            ExistingFilesystem,
            InstanceServiceClient,
            UpdateInstanceRequest,
        )

        client = InstanceServiceClient(channel)
        upd = UpdateInstanceRequest()
        upd.metadata.id = "foo-bar"
        upd.spec.filesystems = [
            AttachedFilesystemSpec(
                attach_mode=AttachedFilesystemSpec.AttachMode.READ_WRITE,
                mount_tag="/mnt/foo-bar",
                existing_filesystem=ExistingFilesystem(
                    id="foo-bar",
                ),
            )
        ]
        upd.spec.filesystems = []
        req = client.update(upd)

        # Await response and metadata
        ret = await req
        assert isinstance(ret, Operation)
    finally:
        # Clean up
        if channel is not None:
            await channel.close()
        await srv.stop(0)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/mk8s/v1/progress_data.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"nebius/mk8s/v1/progress_data.proto\x12\x0enebius.mk8s.v1\"C\n\x0cProgressData\x12\x33\n\x08problems\x18\x01 \x03(\x0b\x32\x17.nebius.mk8s.v1.ProblemR\x08problems\"9\n\x07Problem\x12\x14\n\x05stage\x18\x01 \x01(\tR\x05stage\x12\x18\n\x07message\x18\x02 \x01(\tR\x07messageBZ\n\x15\x61i.nebius.pub.mk8s.v1B\x11ProgressDataProtoP\x01Z,github.com/nebius/gosdk/proto/nebius/mk8s/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.mk8s.v1.progress_data_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025ai.nebius.pub.mk8s.v1B\021ProgressDataProtoP\001Z,github.com/nebius/gosdk/proto/nebius/mk8s/v1'
  _globals['_PROGRESSDATA']._serialized_start=54
  _globals['_PROGRESSDATA']._serialized_end=121
  _globals['_PROBLEM']._serialized_start=123
  _globals['_PROBLEM']._serialized_end=180
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/s3_events.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/storage/v1/s3_events.proto\x12\x11nebius.storage.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\xff\x01\n(CompleteMultipartUploadRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12V\n\x05parts\x18\x03 \x03(\x0b\x32@.nebius.storage.v1.CompleteMultipartUploadRequestParameters.PartR\x05parts\x1a;\n\x04Part\x12\x1f\n\x0bpart_number\x18\x01 \x01(\x05R\npartNumber\x12\x12\n\x04\x65tag\x18\x02 \x01(\tR\x04\x65tag\"u\n\x1f\x43ompleteMultipartUploadResponse\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x12\n\x04\x65tag\x18\x03 \x01(\tR\x04\x65tag\"\xe9\x02\n\x1b\x43opyObjectRequestParameters\x12M\n\x06source\x18\x01 \x01(\x0b\x32\x35.nebius.storage.v1.CopyObjectRequestParameters.SourceR\x06source\x12M\n\x06target\x18\x02 \x01(\x0b\x32\x35.nebius.storage.v1.CopyObjectRequestParameters.TargetR\x06target\x1a\x62\n\x06Source\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\x1aH\n\x06Target\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\"i\n\x12\x43opyObjectResponse\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12?\n\rlast_modified\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0clastModified\"y\n\x1d\x44\x65leteObjectRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\";\n\x14\x44\x65leteObjectResponse\x12#\n\rdelete_marker\x18\x01 \x01(\x08R\x0c\x64\x65leteMarker\"\xd1\x01\n\x1e\x44\x65leteObjectsRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12U\n\x07objects\x18\x04 \x03(\x0b\x32;.nebius.storage.v1.DeleteObjectsRequestParameters.ObjectKeyR\x07objects\x1a\x37\n\tObjectKey\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\"\x9c\x03\n\x15\x44\x65leteObjectsResponse\x12P\n\x07\x64\x65leted\x18\x01 \x03(\x0b\x32\x36.nebius.storage.v1.DeleteObjectsResponse.DeleteSuccessR\x07\x64\x65leted\x12L\n\x06\x65rrors\x18\x02 \x03(\x0b\x32\x34.nebius.storage.v1.DeleteObjectsResponse.DeleteErrorR\x06\x65rrors\x1am\n\rDeleteSuccess\x12#\n\rdelete_marker\x18\x01 \x01(\x08R\x0c\x64\x65leteMarker\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\x1at\n\x0b\x44\x65leteError\x12\x12\n\x04\x63ode\x18\x01 \x01(\tR\x04\x63ode\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x1d\n\nobject_key\x18\x03 \x01(\tR\tobjectKey\x12\x18\n\x07version\x18\x04 \x01(\tR\x07version\"s\n\x17ObjectRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\"\x7f\n\x0eObjectResponse\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\x12?\n\rlast_modified\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0clastModified\"\xdd\x01\n#ListObjectVersionsRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x16\n\x06prefix\x18\x02 \x01(\tR\x06prefix\x12\x1d\n\nkey_marker\x18\x03 \x01(\tR\tkeyMarker\x12*\n\x11version_id_marker\x18\x04 \x01(\x05R\x0fversionIdMarker\x12\x14\n\x05limit\x18\x05 \x01(\x05R\x05limit\x12\x1c\n\tdelimiter\x18\x06 \x01(\tR\tdelimiter\"\x9e\x06\n\x1aListObjectVersionsResponse\x12U\n\x08versions\x18\x01 \x03(\x0b\x32\x39.nebius.storage.v1.ListObjectVersionsResponse.VersionViewR\x08versions\x12\x65\n\x0e\x64\x65lete_markers\x18\x02 \x03(\x0b\x32>.nebius.storage.v1.ListObjectVersionsResponse.DeleteMarkerViewR\rdeleteMarkers\x12\'\n\x0f\x63ommon_prefixes\x18\x03 \x03(\tR\x0e\x63ommonPrefixes\x12\x1c\n\ttruncated\x18\x04 \x01(\x08R\ttruncated\x12&\n\x0fnext_key_marker\x18\x05 \x01(\tR\rnextKeyMarker\x12\x33\n\x16next_version_id_marker\x18\x06 \x01(\x05R\x13nextVersionIdMarker\x1a\xf1\x01\n\x0bVersionView\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12\x16\n\x06latest\x18\x02 \x01(\x08R\x06latest\x12\x1d\n\nobject_key\x18\x03 \x01(\tR\tobjectKey\x12?\n\rlast_modified\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0clastModified\x12\x12\n\x04size\x18\x05 \x01(\x03R\x04size\x12\x1d\n\nversion_id\x18\x06 \x01(\tR\tversionId\x12#\n\rstorage_class\x18\x07 \x01(\tR\x0cstorageClass\x1a\xa9\x01\n\x10\x44\x65leteMarkerView\x12\x16\n\x06latest\x18\x01 \x01(\x08R\x06latest\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12?\n\rlast_modified\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0clastModified\x12\x1d\n\nversion_id\x18\x04 \x01(\tR\tversionId\"\xac\x01\n\x1cListObjectsRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x16\n\x06prefix\x18\x02 \x01(\tR\x06prefix\x12\x1c\n\tdelimiter\x18\x03 \x01(\tR\tdelimiter\x12\x1f\n\x0bstart_after\x18\x04 \x01(\tR\nstartAfter\x12\x14\n\x05limit\x18\x05 \x01(\x05R\x05limit\"\xe5\x02\n\x13ListObjectsResponse\x12K\n\x07objects\x18\x01 \x03(\x0b\x32\x31.nebius.storage.v1.ListObjectsResponse.ObjectViewR\x07objects\x12\'\n\x0f\x63ommon_prefixes\x18\x02 \x03(\tR\x0e\x63ommonPrefixes\x12\x1c\n\ttruncated\x18\x03 \x01(\x08R\ttruncated\x1a\xb9\x01\n\nObjectView\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12?\n\rlast_modified\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0clastModified\x12\x12\n\x04size\x18\x04 \x01(\x03R\x04size\x12#\n\rstorage_class\x18\x05 \x01(\tR\x0cstorageClass\"q\n\x1bPostObjectRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x12\n\x04size\x18\x03 \x01(\x03R\x04size\"B\n\x12PostObjectResponse\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12\x18\n\x07version\x18\x02 \x01(\tR\x07version\"\xb8\x01\n\x1aPutObjectRequestParameters\x12\x1f\n\x0b\x62ucket_name\x18\x01 \x01(\tR\nbucketName\x12\x1d\n\nobject_key\x18\x02 \x01(\tR\tobjectKey\x12\x12\n\x04size\x18\x03 \x01(\x03R\x04size\x12#\n\rstorage_class\x18\x04 \x01(\tR\x0cstorageClass\x12!\n\x0c\x65xpected_md5\x18\x05 \x01(\tR\x0b\x65xpectedMd5\"A\n\x11PutObjectResponse\x12\x12\n\x04\x65tag\x18\x01 \x01(\tR\x04\x65tag\x12\x18\n\x07version\x18\x02 \x01(\tR\x07versionB\\\n\x18\x61i.nebius.pub.storage.v1B\rS3EventsProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.s3_events_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\rS3EventsProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _globals['_COMPLETEMULTIPARTUPLOADREQUESTPARAMETERS']._serialized_start=90
  _globals['_COMPLETEMULTIPARTUPLOADREQUESTPARAMETERS']._serialized_end=345
  _globals['_COMPLETEMULTIPARTUPLOADREQUESTPARAMETERS_PART']._serialized_start=286
  _globals['_COMPLETEMULTIPARTUPLOADREQUESTPARAMETERS_PART']._serialized_end=345
  _globals['_COMPLETEMULTIPARTUPLOADRESPONSE']._serialized_start=347
  _globals['_COMPLETEMULTIPARTUPLOADRESPONSE']._serialized_end=464
  _globals['_COPYOBJECTREQUESTPARAMETERS']._serialized_start=467
  _globals['_COPYOBJECTREQUESTPARAMETERS']._serialized_end=828
  _globals['_COPYOBJECTREQUESTPARAMETERS_SOURCE']._serialized_start=656
  _globals['_COPYOBJECTREQUESTPARAMETERS_SOURCE']._serialized_end=754
  _globals['_COPYOBJECTREQUESTPARAMETERS_TARGET']._serialized_start=756
  _globals['_COPYOBJECTREQUESTPARAMETERS_TARGET']._serialized_end=828
  _globals['_COPYOBJECTRESPONSE']._serialized_start=830
  _globals['_COPYOBJECTRESPONSE']._serialized_end=935
  _globals['_DELETEOBJECTREQUESTPARAMETERS']._serialized_start=937
  _globals['_DELETEOBJECTREQUESTPARAMETERS']._serialized_end=1058
  _globals['_DELETEOBJECTRESPONSE']._serialized_start=1060
  _globals['_DELETEOBJECTRESPONSE']._serialized_end=1119
  _globals['_DELETEOBJECTSREQUESTPARAMETERS']._serialized_start=1122
  _globals['_DELETEOBJECTSREQUESTPARAMETERS']._serialized_end=1331
  _globals['_DELETEOBJECTSREQUESTPARAMETERS_OBJECTKEY']._serialized_start=1276
  _globals['_DELETEOBJECTSREQUESTPARAMETERS_OBJECTKEY']._serialized_end=1331
  _globals['_DELETEOBJECTSRESPONSE']._serialized_start=1334
  _globals['_DELETEOBJECTSRESPONSE']._serialized_end=1746
  _globals['_DELETEOBJECTSRESPONSE_DELETESUCCESS']._serialized_start=1519
  _globals['_DELETEOBJECTSRESPONSE_DELETESUCCESS']._serialized_end=1628
  _globals['_DELETEOBJECTSRESPONSE_DELETEERROR']._serialized_start=1630
  _globals['_DELETEOBJECTSRESPONSE_DELETEERROR']._serialized_end=1746
  _globals['_OBJECTREQUESTPARAMETERS']._serialized_start=1748
  _globals['_OBJECTREQUESTPARAMETERS']._serialized_end=1863
  _globals['_OBJECTRESPONSE']._serialized_start=1865
  _globals['_OBJECTRESPONSE']._serialized_end=1992
  _globals['_LISTOBJECTVERSIONSREQUESTPARAMETERS']._serialized_start=1995
  _globals['_LISTOBJECTVERSIONSREQUESTPARAMETERS']._serialized_end=2216
  _globals['_LISTOBJECTVERSIONSRESPONSE']._serialized_start=2219
  _globals['_LISTOBJECTVERSIONSRESPONSE']._serialized_end=3017
  _globals['_LISTOBJECTVERSIONSRESPONSE_VERSIONVIEW']._serialized_start=2604
  _globals['_LISTOBJECTVERSIONSRESPONSE_VERSIONVIEW']._serialized_end=2845
  _globals['_LISTOBJECTVERSIONSRESPONSE_DELETEMARKERVIEW']._serialized_start=2848
  _globals['_LISTOBJECTVERSIONSRESPONSE_DELETEMARKERVIEW']._serialized_end=3017
  _globals['_LISTOBJECTSREQUESTPARAMETERS']._serialized_start=3020
  _globals['_LISTOBJECTSREQUESTPARAMETERS']._serialized_end=3192
  _globals['_LISTOBJECTSRESPONSE']._serialized_start=3195
  _globals['_LISTOBJECTSRESPONSE']._serialized_end=3552
  _globals['_LISTOBJECTSRESPONSE_OBJECTVIEW']._serialized_start=3367
  _globals['_LISTOBJECTSRESPONSE_OBJECTVIEW']._serialized_end=3552
  _globals['_POSTOBJECTREQUESTPARAMETERS']._serialized_start=3554
  _globals['_POSTOBJECTREQUESTPARAMETERS']._serialized_end=3667
  _globals['_POSTOBJECTRESPONSE']._serialized_start=3669
  _globals['_POSTOBJECTRESPONSE']._serialized_end=3735
  _globals['_PUTOBJECTREQUESTPARAMETERS']._serialized_start=3738
  _globals['_PUTOBJECTREQUESTPARAMETERS']._serialized_end=3922
  _globals['_PUTOBJECTRESPONSE']._serialized_start=3924
  _globals['_PUTOBJECTRESPONSE']._serialized_end=3989
# @@protoc_insertion_point(module_scope)



"""
End-to-end test for storage bucket operations using Nebius SDK.

This test requires NEBIUS_E2E_CONFIG_FILE environment variable to be set
to a valid YAML config file. If the environment variable is not set or
the file is invalid, the test will be skipped.
"""

import base64
import binascii
import os
import random
import string
import tempfile
from pathlib import Path

import pytest
import yaml

from nebius.aio.cli_config import Config
from nebius.api.nebius.common.v1 import ResourceMetadata
from nebius.api.nebius.storage.v1 import (
    BucketServiceClient,
    BucketSpec,
    CreateBucketRequest,
    DeleteBucketRequest,
    GetBucketRequest,
    ListBucketsRequest,
    VersioningPolicy,
)
from nebius.sdk import SDK


def _generate_random_string(length: int = 8) -> str:
    """Generate a random string of lowercase letters and digits."""
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))


def _get_config_file_path() -> tuple[str | None, bool]:
    """
    Get the config file path from environment variable.

    Returns:
        tuple: (config_file_path, is_temp_file)
        - config_file_path: Path to the config file or None if not available
        - is_temp_file: True if the file was created from base64 content
    """
    # First try NEBIUS_E2E_CONFIG_FILE
    config_file = os.environ.get("NEBIUS_E2E_CONFIG_FILE")
    if config_file and Path(config_file).exists():
        return config_file, False

    # If file env var not found or file doesn't exist, try base64 variant
    config_b64 = os.environ.get("NEBIUS_E2E_CONFIG_B64")
    if not config_b64:
        return None, False

    try:
        # Decode base64 content
        config_content = base64.b64decode(config_b64).decode("utf-8")

        # Validate that it's valid YAML
        yaml.safe_load(config_content)

        # Create temporary file
        temp_file = tempfile.NamedTemporaryFile(
            mode="w", suffix=".yaml", prefix="nebius_e2e_config_", delete=False
        )
        temp_file.write(config_content)
        temp_file.close()

        return temp_file.name, True

    except (binascii.Error, UnicodeDecodeError, yaml.YAMLError, OSError):
        return None, False


def _validate_yaml_config(config_file: str) -> bool:
    """Validate that the config file is a valid YAML."""
    try:
        with open(config_file, "r") as f:
            yaml.safe_load(f)
        return True
    except (yaml.YAMLError, OSError):
        return False


@pytest.mark.asyncio
async def test_e2e_storage_bucket_lifecycle() -> None:
    """
    End-to-end test for storage bucket lifecycle:
    1. Create a storage bucket with specific configuration
    2. Get the bucket by ID
    3. List buckets and verify our bucket is present
    4. Delete the bucket
    """
    # Check if config file is set and valid
    config_file, is_temp_file = _get_config_file_path()
    try:
        if not config_file:
            pytest.skip(
                "Neither NEBIUS_E2E_CONFIG_FILE nor NEBIUS_E2E_CONFIG_B64 "
                "environment variable is set or valid"
            )

        if not is_temp_file and not _validate_yaml_config(config_file):
            pytest.skip("NEBIUS_E2E_CONFIG_FILE does not point to a valid YAML file")

        # Create SDK from config
        config = Config(config_file=config_file)
        sdk = SDK(
            config_reader=config,
            user_agent_prefix="pysdk-e2e-test/1.0 (bucket-storage)",
        )

        # Get parent ID for assertions
        parent_id = config.parent_id
        assert parent_id is not None, "Parent ID should not be None"
        assert parent_id == sdk.parent_id(), "Parent ID should match SDK's parent ID"

        # Generate unique bucket name
        random_suffix = _generate_random_string()
        bucket_name = f"python-e2e-{random_suffix}"

        bucket_service = BucketServiceClient(sdk)
        bucket_id = None

        async with sdk:
            try:
                # Create bucket
                create_request = CreateBucketRequest(
                    metadata=ResourceMetadata(
                        name=bucket_name,
                    ),
                    spec=BucketSpec(
                        versioning_policy=VersioningPolicy.DISABLED,
                        max_size_bytes=4096,
                    ),
                )

                create_operation = await bucket_service.create(create_request)
                await create_operation.wait()
                bucket_id = create_operation.resource_id

                assert (
                    bucket_id is not None
                ), "Bucket creation should return a valid bucket ID"

                # Get bucket by ID
                get_request = GetBucketRequest(id=bucket_id)
                bucket = await bucket_service.get(get_request)

                assert bucket.metadata.id == bucket_id
                assert bucket.metadata.name == bucket_name
                assert bucket.spec.versioning_policy == VersioningPolicy.DISABLED
                assert bucket.spec.max_size_bytes == 4096

                # List buckets and find our bucket
                list_request = ListBucketsRequest(parent_id=parent_id)
                buckets_response = await bucket_service.list(list_request)

                # Find our bucket in the list
                found_bucket = None
                for bucket_item in buckets_response.items:
                    if bucket_item.metadata.id == bucket_id:
                        found_bucket = bucket_item
                        break

                assert (
                    found_bucket is not None
                ), f"Bucket {bucket_id} should be found in the list"
                assert found_bucket.metadata.name == bucket_name

                delete_request = DeleteBucketRequest(id=bucket_id)
                delete_operation = await bucket_service.delete(delete_request)
                await delete_operation.wait()
                bucket_id = None  # Reset bucket_id after deletion

            except Exception as e:
                # If test fails, still try to cleanup
                if bucket_id:
                    try:
                        delete_request = DeleteBucketRequest(id=bucket_id)
                        delete_operation = await bucket_service.delete(delete_request)
                        await delete_operation.wait()
                    except Exception as cleanup_error:
                        # Log cleanup error but don't fail the test
                        print(
                            "Warning: Failed to cleanup bucket "
                            f"{bucket_id}: {cleanup_error}"
                        )
                raise e

    finally:
        # Cleanup temporary config file if created from base64
        if is_temp_file and config_file and Path(config_file).exists():
            try:
                os.unlink(config_file)
            except OSError:
                pass  # Ignore cleanup errors for temp file


if __name__ == "__main__":
    # Allow running the test directly for debugging
    import asyncio

    async def main() -> None:
        await test_e2e_storage_bucket_lifecycle()

    asyncio.run(main())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/tenant_user_account_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v1 import tenant_user_account_pb2 as nebius_dot_iam_dot_v1_dot_tenant__user__account__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/nebius/iam/v1/tenant_user_account_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a nebius/common/v1/operation.proto\x1a\'nebius/iam/v1/tenant_user_account.proto\"-\n\x1bGetTenantUserAccountRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xa8\x01\n\x1dListTenantUserAccountsRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12 \n\tpage_size\x18\x02 \x01(\x03H\x00R\x08pageSize\x88\x01\x01\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x1b\n\x06\x66ilter\x18\x04 \x01(\tB\x03\xc0J\x01R\x06\x66ilterB\x0c\n\n_page_size\"\x80\x01\n\x1eListTenantUserAccountsResponse\x12\x36\n\x05items\x18\x01 \x03(\x0b\x32 .nebius.iam.v1.TenantUserAccountR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"/\n\x1d\x42lockTenantUserAccountRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"1\n\x1fUnblockTenantUserAccountRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id2\x8c\x03\n\x18TenantUserAccountService\x12S\n\x03Get\x12*.nebius.iam.v1.GetTenantUserAccountRequest\x1a .nebius.iam.v1.TenantUserAccount\x12\x63\n\x04List\x12,.nebius.iam.v1.ListTenantUserAccountsRequest\x1a-.nebius.iam.v1.ListTenantUserAccountsResponse\x12R\n\x05\x42lock\x12,.nebius.iam.v1.BlockTenantUserAccountRequest\x1a\x1b.nebius.common.v1.Operation\x12V\n\x07Unblock\x12..nebius.iam.v1.UnblockTenantUserAccountRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBd\n\x14\x61i.nebius.pub.iam.v1B\x1dTenantUserAccountServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.tenant_user_account_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\035TenantUserAccountServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _LISTTENANTUSERACCOUNTSREQUEST.fields_by_name['filter']._options = None
  _LISTTENANTUSERACCOUNTSREQUEST.fields_by_name['filter']._serialized_options = b'\300J\001'
  _TENANTUSERACCOUNTSERVICE._options = None
  _TENANTUSERACCOUNTSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_GETTENANTUSERACCOUNTREQUEST']._serialized_start=167
  _globals['_GETTENANTUSERACCOUNTREQUEST']._serialized_end=212
  _globals['_LISTTENANTUSERACCOUNTSREQUEST']._serialized_start=215
  _globals['_LISTTENANTUSERACCOUNTSREQUEST']._serialized_end=383
  _globals['_LISTTENANTUSERACCOUNTSRESPONSE']._serialized_start=386
  _globals['_LISTTENANTUSERACCOUNTSRESPONSE']._serialized_end=514
  _globals['_BLOCKTENANTUSERACCOUNTREQUEST']._serialized_start=516
  _globals['_BLOCKTENANTUSERACCOUNTREQUEST']._serialized_end=563
  _globals['_UNBLOCKTENANTUSERACCOUNTREQUEST']._serialized_start=565
  _globals['_UNBLOCKTENANTUSERACCOUNTREQUEST']._serialized_end=614
  _globals['_TENANTUSERACCOUNTSERVICE']._serialized_start=617
  _globals['_TENANTUSERACCOUNTSERVICE']._serialized_end=1013
# @@protoc_insertion_point(module_scope)
